if exists ( select * from INFORMATION_SCHEMA.ROUTINES where ROUTINE_NAME = 'LeaseQueuedFlows')
	drop procedure LeaseQueuedFlows;
go
create procedure LeaseQueuedFlows( @platformServerId varchar(254), @pickUpCount int)
as
begin
	-- calculation queuedDateTo
	declare @queuedDateTo datetime;
	declare @PickupItemsOlderThan float;
	declare @currentDate datetime;
	declare @machineBy nvarchar(254), @ip nvarchar(254);
	
	set @currentDate =GETDATE();
	
	set @PickupItemsOlderThan = (
		select top 1 only_older_than from work_queue_worker 
			inner join platform_server on work_queue_worker.entity_name = COALESCE(platform_server.worker_name,'Default Worker')  and platform_server.platform_server_id = @platformServerId);

	set @machineBy = (
		select top 1 entity_name from platform_server 
			where platform_server_id = @platformServerId);
	set @ip = (
		select top 1 ip_address from platform_server 
			where platform_server_id = @platformServerId);

	
	set @queuedDateTo = DATEADD( MILLISECOND, - @PickupItemsOlderThan, @currentDate);
	if ((select top 1 run_specific_queues from platform_server where platform_server_id = @platformServerId) = 1)
		with queuedflows as(
			select top (@pickUpCount) flow.queued_flow_id, q.lease_time from queued_flow flow
				inner join work_queue q on flow.work_queue_id = q.work_queue_id 
				inner join platform_server_queues psq on psq.platform_server_id = @platformServerId and q.work_queue_id= psq.platform_server_queue_id
				where 
				(leased_until is null or  leased_Until < @currentDate) and 
				queued_when < @queuedDateTo and
				(ended_in_error is null or ended_in_error = 0)
				order by queued_when)
		update queued_flow  
			set leased_until = DATEADD( MILLISECOND, queuedflows.lease_time, @currentDate),
				leased_by_machine = @machineBy
		output inserted.*
		from queued_flow, queuedflows where  queued_flow.queued_flow_id = queuedflows.queued_flow_id;
	else
		with queuedflows as(
			select top (@pickUpCount) flow.queued_flow_id, q.lease_time from queued_flow flow
				inner join work_queue q on flow.work_queue_id = q.work_queue_id 
				where 
				(leased_until is null or  leased_Until < @currentDate) and
				(ended_in_error is null or ended_in_error = 0)and
				queued_when < @queuedDateTo and
				(ended_in_error is null or ended_in_error = 0)
				order by queued_when)
		update queued_flow  
			set leased_until = DATEADD( MILLISECOND, queuedflows.lease_time, @currentDate),
				leased_by_machine = @machineBy,
				leased_by_ip = @ip
		output inserted.*
		from queued_flow, queuedflows where  queued_flow.queued_flow_id = queuedflows.queued_flow_id;
end
