create or replace procedure LeaseQueuedFlows(
  platformServerId varchar2, 
  pickUpCount int, 
  p_cursor out sys_refcursor )
as
	queuedDateTo date;
	PickupItemsOlderThan float;
	currentDate date := SYSDATE;
  flow_set ArrayOfIdType;
  runSpecificQueues number(1,0);
  machineBy nvarchar2(254);
  ip nvarchar2(254);
begin

	select only_older_than into PickupItemsOlderThan from work_queue_worker 
		inner join platform_server on 
      work_queue_worker.entity_name = COALESCE(platform_server.worker_name,N'Default Worker') and 
      platform_server.platform_server_id = platformServerId
  where rownum = 1;
    
  
  select run_specific_queues, entity_name, ip_address into runSpecificQueues, machineBy, ip
    from platform_server where platform_server.platform_server_id = platformServerId and rownum = 1;

  queuedDateTo :=  currentDate + NUMTODSINTERVAL (-PickupItemsOlderThan/1000,' SECOND');
  IF runSpecificQueues = 1 THEN
      update queued_flow qf
    set   leased_by_machine = machineBy,
    leased_by_ip = ip,
    leased_until = ( 
      with limited_queued_flow as 
        (select * from (
            select flow.queued_flow_id, q.lease_time  from queued_flow flow
              inner join work_queue q on flow.work_queue_id = q.work_queue_id
              inner join platform_server_queues psq on psq.platform_server_id = platformServerId and q.work_queue_id= psq.platform_server_queue_id
            where 
              (leased_until is null or  leased_Until < currentDate) and
              queued_when < queuedDateTo and
              (ended_in_error is null or ended_in_error = 0)
              order by queued_when) 
          where rownum <= pickUpCount
          ) 
        select currentDate + NUMTODSINTERVAL (limited_queued_flow.lease_time/1000,' SECOND')  
        from limited_queued_flow 
        where limited_queued_flow.queued_flow_id = qf.queued_flow_id)
     where exists(
      with limited_queued_flow as 
        (select * from (
            select flow.queued_flow_id, q.lease_time  from queued_flow flow
              inner join work_queue q on flow.work_queue_id = q.work_queue_id 
              inner join platform_server_queues psq on psq.platform_server_id = platformServerId and q.work_queue_id= psq.platform_server_queue_id            
          where 
              (leased_until is null or  leased_Until < currentDate) and
              queued_when < queuedDateTo and
              (ended_in_error is null or ended_in_error = 0)
              order by queued_when) 
          where rownum <= pickUpCount
          ) 
        select * from limited_queued_flow 
        where limited_queued_flow.queued_flow_id = qf.queued_flow_id)
    returning qf.queued_flow_id bulk collect into flow_set;
  ELSE
   update queued_flow qf
    set   leased_by_machine = machineBy,
    leased_until = ( 
      with limited_queued_flow as 
        (select * from (
            select flow.queued_flow_id, q.lease_time  from queued_flow flow
              inner join work_queue q on flow.work_queue_id = q.work_queue_id 
            where 
              (leased_until is null or  leased_Until < currentDate) and
              queued_when < queuedDateTo and
              (ended_in_error is null or ended_in_error = 0)
              order by queued_when) 
          where rownum <= pickUpCount
          ) 
        select currentDate + NUMTODSINTERVAL (limited_queued_flow.lease_time/1000,' SECOND')  
        from limited_queued_flow 
        where limited_queued_flow.queued_flow_id = qf.queued_flow_id)
     where exists(
      with limited_queued_flow as 
        (select * from (
            select flow.queued_flow_id, q.lease_time  from queued_flow flow
              inner join work_queue q on flow.work_queue_id = q.work_queue_id 
            where 
              (leased_until is null or  leased_Until < currentDate) and
              queued_when < queuedDateTo and
              (ended_in_error is null or ended_in_error = 0)
              order by queued_when) 
          where rownum <= pickUpCount
          ) 
        select * from limited_queued_flow 
        where limited_queued_flow.queued_flow_id = qf.queued_flow_id)
    returning qf.queued_flow_id bulk collect into flow_set;
  commit;
  END IF;
  
  OPEN p_cursor FOR 
    select q.* from queued_flow q, table(flow_set) t
        where value(t) = q.queued_flow_id;
end;

